@@ -1,5 +1,5 @@
 preprocess.ffdf <-
-function(dat, colNames, sortCol,  decreasing, scales, palet, nBins, from, to) {
+function(dat, datName, colNames, sortCol,  decreasing, scales, pals, nBins, from, to) {
    if (!require(ff)){
 		stop("This function needs package ff")
    }   
@@ -9,15 +9,15 @@ function(dat, colNames, sortCol,  decreasing, scales, palet, nBins, from, to) {
 	#############################
 	## Determine column classes
 	#############################
-	
-	for (i in which(vmode(dat) == "logical")) {
+	isLogical <- vmode(dat) == "logical"
+	for (i in which(isLogical)) {
 		levels(dat[,i]) <- c("TRUE", "FALSE")
 	}
 	
 	isFactor <- sapply(physical(dat), is.factor)	
 	
 	## find numerical variables
-	isNumber <- !isFactor
+	isNumber <- !isFactor & !isLogical
 	
 	## cast logical columns to factors
 	#TODO
@@ -84,82 +84,81 @@ function(dat, colNames, sortCol,  decreasing, scales, palet, nBins, from, to) {
 		
 		numcols <- names(dat)[as.which(isNumber)] # needed because isNumber is otherwise recycled!!
 		
-		## calculate means by dividing by binSizes and sum
-		ncwmean <- function(df){
-		   size <- binSizes[df$aggIndex]
-		   l <- lapply( df[,numcols]
-		              , function(x){
-					       sum(x/size, na.rm=TRUE)
-						}
-					  )
-		   names(l) <- numcols
-		   as.data.frame(l)
-		}
-
-		ncomplete <- function(df){
-		   size <- binSizes[df$aggIndex[1]] #otherwise multiple output...
-		   l <- lapply( (df[numcols])
-		              , function(x){
-					      100*sum(!is.na(x))/size
-						}
-					  )
-		   names(l) <- numcols
-		   as.data.frame(l)
-		}
-		
-		datMean <- datCompl <- NULL
+		datSum <- datCompl <- NULL
 
 		## bypass Rcmd warning
 		aggIndex <- NULL; rm(aggIndex)
 
+		.SD <- NULL; rm(.SD); #trick R CMD check
 		for (i in chunk(dat)){
-			cdat <- dat[i,]
-			dmean <- ddply(cdat, .(aggIndex), ncwmean)
-			datMean <- rbind(datMean,dmean)
+			cdat <- data.table(dat[i,])[, c(numcols, "aggIndex"), with=FALSE]
+			setkey(cdat, aggIndex)
+			dsum <- cdat[, lapply(.SD, function(x)sum(x, na.rm=TRUE)), by=aggIndex]
+
+			datSum <- rbind(datSum, dsum)
 			## calculate completion percentages
-			dcompl <- ddply(cdat, .(aggIndex), ncomplete)
+			dcompl <- cdat[, lapply(.SD, function(x)sum(!is.na(x))), by=aggIndex]
+
 			datCompl <- rbind(datCompl, dcompl)
 		}
 
-		datMean <- ddply(datMean, .(aggIndex), colwise(sum, numcols))
-		datCompl <- ddply(datCompl, .(aggIndex), colwise(sum, numcols))
+		datSum <- datSum[, lapply(.SD, sum), by=aggIndex][!is.na(aggIndex),]
+		datSum$binSizes <- binSizes[datSum$aggIndex]
 		
-		datMean <- datMean[-(nBins+1),-1, drop=FALSE]
-		datCompl <- datCompl[-(nBins+1),-1, drop=FALSE]
-		#datCompl <- as.data.frame(apply(datCompl, 2, function(x,y){100-floor(x/y*100)}, binSizes))
-		## set means of bins with all missings to 0
-		#datMean[datCompl==0] <- 0
+		datCompl <- datCompl[, lapply(.SD, sum), by=aggIndex][!is.na(aggIndex),]
+		datCompl$binSizes <- binSizes[datSum$aggIndex]
+		datCompl <- data.table(lapply(datCompl[, numcols, with=FALSE], FUN="/", datCompl$binSizes))
+
+		datMean <- datSum[, numcols, with=FALSE] / datCompl[, numcols, with=FALSE]
+		datMean <- data.table(lapply(datMean, FUN="/", datSum$binSizes))
+	
+		datCompl <- 100 * datCompl
+
 	}
 	#####################
 	## Aggregate categorical variables
 	#####################
 	if (any(!isNumber)) {
-		freq <- list()
-		for (i in chunk(dat)){
-		   cdat <- dat[i,]
-		   for (catCol in (names(cdat)[!isNumber])){
-		      tab <- freq[[catCol]]
-			  if(is.null(tab)) {
-				tab <- as.matrix(table(cdat$aggIndex,cdat[[catCol]], useNA="always"))
-			  }
-			  else {
-				tab <- tab + as.matrix(table(cdat$aggIndex,cdat[[catCol]], useNA=c("always")))
-			  }
-		      freq[[catCol]] <- tab
-		   }
-		}
-		datFreq <- lapply( freq
-		                 , function(x){
-						      naCol <- ncol(x)
-						      colnames(x)[naCol] <- "missing"
-							  if (max(x[,naCol]) == 0){ #drop nacol
-							     x <- x[,-naCol]
-							  }
-							  list(freqTable=x[1:nBins,], categories=colnames(x))      
-		                   })
+		catcols <- names(dat)[as.which(!isNumber)] # needed because isNumber is otherwise recycled!!
+
+
+		datFreq <- list()
+		maxlevels <- max(sapply(dat[1,], nlevels))
 		
-		#datFreq <- lapply(dat[,][!isNumber], FUN=getFreqTable, dat$aggIndex[], nBins)
-	}
+		chunks <- chunk(dat)
+		
+		
+		cdat <- dat[chunks[[1]], c(catcols, "aggIndex")]
+		# cast logicals to factors
+		for (i in colNames[isLogical]) {
+			cdat[[i]] <- factor(cdat[[i]], levels=c("TRUE", "FALSE"))
+		}
+
+		datFreq <- lapply(cdat[, catcols], FUN=getFreqTable_DT, cdat$aggIndex, nBins, useNA="always")
+	
+		for (i in chunks[-1]){
+			cdat <- dat[i, c(catcols, "aggIndex")]
+			# cast logicals to factors
+			for (i in colNames[isLogical]) {
+				cdat[[i]] <- factor(cdat[[i]], levels=c("TRUE", "FALSE"))
+			}
+
+			datFreq2 <- lapply(cdat[, catcols], FUN=getFreqTable_DT, cdat$aggIndex, nBins, useNA="always")
+
+			datFreq <- mapply(datFreq, datFreq2, FUN=function(df1, df2){
+					return(list(freqTable=df1$freqTable + df2$freqTable, categories=df1$categories))
+				}, SIMPLIFY=FALSE)
+		}
+
+		datFreq <- mapply(datFreq, FUN=function(df) {
+				if (all(df$freqTable[,"missing"]==0)) {
+					ncols <- ncol(df$freqTable)
+					df$freqTable <- df$freqTable[,-ncols]
+					df$categories <- df$categories[-ncols]
+				}
+				return(df)
+			}, SIMPLIFY=FALSE)
+	}	
 	
 	#############################
 	##
@@ -167,15 +166,16 @@ function(dat, colNames, sortCol,  decreasing, scales, palet, nBins, from, to) {
 	##
 	#############################
 
+	
 	tab <- list()
+	tab$dataset <- datName
 	tab$n <- n
 	tab$nBins <- nBins
 	tab$binSizes <- binSizes
-	tab$scales <- scales
 	tab$isNumber <- isNumber
 	## tab$row contains info about bins/y-axis
 	tab$rows <- list( heights = -(binSizes/vp$m)
-	                , y = 1 - c(0,cumsum(binSizes/vp$m)[-nBins])
+	                , y = 1- c(0,cumsum(binSizes/vp$m)[-nBins])
 	                , m = vp$m
 	                , from = from
 	                , to = to
@@ -185,21 +185,28 @@ function(dat, colNames, sortCol,  decreasing, scales, palet, nBins, from, to) {
 	## create column list
 	tab$columns <- list()
 	paletNr <- 1
+	scales <- rep(scales, length.out=sum(isNumber))
+	scalesNr <- 1
 	for (i in 1:n) {
 		sortc <- ifelse(i %in% sortCol, ifelse(decreasing[which(i==sortCol)], "decreasing", "increasing"), "")
 		col <- list(name = colNames[i], isnumeric = isNumber[i], sort=sortc)
 		if (isNumber[i]) {
 			col$mean <- datMean[[colNames[i]]]
 			col$compl <- datCompl[[colNames[i]]]
+			# TODO
+			#col$lower <- datLower[[colNames[i]]]
+			#col$upper <- datUpper[[colNames[i]]]
+			col$scale_init <- scales[scalesNr]
+			scalesNr <- scalesNr + 1
 		} else {
 			col$freq <- datFreq[[colNames[i]]]$freqTable
 			col$categories <- datFreq[[colNames[i]]]$categories
-			col$palet <- palet[paletNr]
-			paletNr <- ifelse(paletNr==length(palet), 1, paletNr + 1)
+			col$paletname <- pals$name[paletNr]
+			col$palet <- pals$palette[[paletNr]]
+			paletNr <- ifelse(paletNr==length(pals), 1, paletNr + 1)
 		}
-		tab$columns[[i]] <- col
+ 		tab$columns[[i]] <- col
 	}
 	
 	return(tab)
-}
-     
\ No newline at end of file
+}
\ No newline at end of file
